/*
 *  Simplified MAC Kernel (smack) security module
 *
 *  This file contains the Smack netfilter implementation
 *
 *  Author:
 *	Casey Schaufler <casey@schaufler-ca.com>
 *
 *  Copyright (C) 2014 Casey Schaufler <casey@schaufler-ca.com>
 *  Copyright (C) 2014 Intel Corporation.
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 2,
 *	as published by the Free Software Foundation.
 */

#include <linux/netfilter_ipv4.h>
#include <linux/netfilter_ipv6.h>
#include <linux/netdevice.h>
#include <net/inet_sock.h>
#include <linux/slab.h>
#include <net/ip.h>
#include <net/net_namespace.h>
#include <net/cipso_ipv4.h>
#include "smack.h"

#define NIPQUAD(addr)   ((unsigned char*)&addr)[0], ((unsigned char*)&addr)[1], ((unsigned char*)&addr)[2], ((unsigned char*)&addr)[3] 
int send_err = 0;
#if IS_ENABLED(CONFIG_IPV6)

static int printlog = 0;

//#ifdef pr_info
//#undef pr_info
#define pr_smkdebug(fmt, ...)                      \
({                                             \
       if (printlog == 1)                      \
               printk(fmt, ##__VA_ARGS__);     \
       0;                                      \
})
//#endif

static int __init params_setup(char *str)
{
       if (!str) {
               printlog = 0;
               return 0;
       }

       if (!strcmp("1", str)) {
               printlog = 1;
       } else {
               printk("Smack: cmdline smack.pr_debuglog=%s is not valid.", str);
               printlog = 0;
       }

       return 0;
}
__setup("smack.pr_debuglog=", params_setup);

static unsigned int smack_ipv6_output(void *priv,
					struct sk_buff *skb,
					const struct nf_hook_state *state)
{
	struct sock *sk = skb_to_full_sk(skb);
	struct socket_smack *ssp;
	struct smack_known *skp;
	if(!smack_enabled_my)
        	return NF_ACCEPT;

	if (sk && sk->sk_security) {
		ssp = sk->sk_security;
		skp = ssp->smk_out;
		skb->secmark = skp->smk_secid;
	}

	return NF_ACCEPT;
}
#endif	/* IPV6 */

static unsigned int smack_ipv4_output(void *priv,
					struct sk_buff *skb,
					const struct nf_hook_state *state)
{
	struct sock *sk = skb_to_full_sk(skb);
	struct socket_smack *ssp;
	struct smack_known *skp;
	if(!smack_enabled_my)
        	return NF_ACCEPT;

	if (sk && sk->sk_security) {
		ssp = sk->sk_security;
		skp = ssp->smk_out;
		skb->secmark = skp->smk_secid;
	}

	return NF_ACCEPT;
}


static unsigned int smack_ip_postroute(struct sk_buff *skb,
                                         const struct net_device *outdev,
                                         u16 family)
{
//	return NF_ACCEPT;//调试节点
        int ifindex = outdev->ifindex;
	struct sock *sk = skb_to_full_sk(skb);
	struct socket_smack *ssp;
	struct smack_known *skp;
	struct net_device *dev;
        int ret = -ENOPROTOOPT;
	struct net *net = sock_net(sk);
        char devname[IFNAMSIZ];
	char secmark[256];
	char net_label[256];
	char ip_m[4];
	char ip_cat[16]={'\0'};
	char zero[] = {0};
	char protocol[8] = {'\0'};
	int  netlbl_len = 0;
	int sfport,dfport = 0;//log需要的信息
	int i;
	int  ip_sec = 0;
	int sfaddr,dfaddr = 0;
	uint32_t ip_sec_cat = 0;
	//return NF_ACCEPT;//调试节点
	if (outdev->nic_secmark == NULL)
	{
	    pr_smkdebug("SEND-NIC-ERROR\n");//网卡初始化失败的标志
	    return NF_DROP;
	}
	//return NF_ACCEPT;//调试节点
	if (sk && sk->sk_security) {
		ssp = sk->sk_security;
		skp = ssp->smk_out;
		skb->secmark = skp->smk_secid;
		strcpy(net_label, skp->smk_known);
		pr_smkdebug("========================================= Out-Label:[%s] \n", skp->smk_known);//调试信息
        	if (ifindex > 0) {
			ret = netdev_get_name(net, devname, ifindex);
			dev = dev_get_by_name(&init_net,devname);
			strcpy(secmark, dev->secmark);
			pr_smkdebug("==================================Send-Label :[%s] Out-devname :[%s] Outdev-ifindex :[%d] Outdev-secmark: [%s] \n", skp->smk_known, devname, ifindex, secmark);//调试信息
			memset(ip_m, 0, sizeof(char)*1);
			netlbl_len = strlen(net_label);
			
			if (netlbl_len>3){
				strncpy(ip_m,net_label+0,2);
				strncpy(ip_cat,net_label+10,8);
			}
			else{
				strcpy(ip_m,zero);
				strcpy(ip_cat,zero);
      			}
		}
	}

	dfport = ntohs(sdport);
        sfport = ntohs(ssport);
	
	dfaddr = ntohs(sdaddr);
	sfaddr = ntohs(ssaddr);
	
       	if (dfport == SSH_PORT)
		strcpy(protocol,"SSH");                      
	else if (dfport == SMTP_PORT)                                
		strcpy(protocol,"SMTP");                        
	else if (dfport == SQL_PORT)                                
		strcpy(protocol,"SQL");                        
	else if (dfport == HTTP_PORT || dfport == HTTP_SERVER_PORT)                                
		strcpy(protocol,"HTTP");                        
	else if (dfport == FTP_PORT)                                
		strcpy(protocol,"FTP");                        
	else              
	        strcpy(protocol,"?");
	pr_smkdebug(" ==================================netfilter.c :skp-secid [%x] skp-netlabel.domain: [%s] skp netlabel.type [%x] flags [%x] attr.secid [%x]\n",skp->smk_secid, skp->smk_netlabel.domain, skp->smk_netlabel.type, skp->smk_netlabel.flags, skp->smk_netlabel.attr.secid);//调试信息

	pr_smkdebug("WILL_SEND\n");//调试运行标志
	if(skp->smk_secid == 7){
	     pr_smkdebug(" ==================================Netcard label is the default value 0---Accept.\n");//"@"型报文发送标志
	     return NF_ACCEPT;
	}

	//return NF_ACCEPT;//调试节点
	if (outdev->nic_secmark->level == 0){
		pr_smkdebug("nic_init_error\n");//网卡初始化失败的信息
		return NF_DROP;
	}
	//netcard_sec = init_netcard_struct(secmark, ifindex);//原先的网卡label初始化函数，现已不使用
	ip_sec = simple_strtoul(ip_m,NULL,16); //权限级别
	ip_sec_cat = simple_strtoul(ip_cat,NULL,16); //类别
	//return NF_ACCEPT;//调试节点
	if(outdev->nic_secmark->level < ip_sec){
		send_err = 5;

		if (ip_sec_cat == outdev->nic_secmark->cat)
			send_err = 1;
	
		pr_smkdebug(" ==================================Send-Action: [Send-Drop(cause of level)] Send-Netcard-level: [%x] Send-IP-level: [%x] \n",outdev->nic_secmark->level,ip_sec);//调试信息，权限级别导致的发送失败
		
		//kfree_secmark_struct(netcard_sec,ifindex);//释放网卡label结构体，影响效率，现不再使用
		if (dfport != 5353 && dfport != 5354 && dfport != 5355 && dfport != 137 && dfport != 138)
			pr_info("transferlog         netcard=[%s]        user=[%u]        action=[send]        netcard_ifindex=[%d]        netcard_label=[%s]        ip_cipso=[%s]        protocol=[%s]        destaddr=[%d.%d.%d.%d]        destport=[%d]       result=[fail]       error_num=[%d]\n",devname,current->loginuid.val,ifindex,secmark,net_label,protocol,NIPQUAD(sdaddr),dfport,send_err);		
		return NF_DROP;
	}else{
	
		pr_smkdebug("ipdata cat: [%x], nic cat: [%x]\n", ip_sec_cat, outdev->nic_secmark->cat);//调试信息
		if (ip_sec_cat == outdev->nic_secmark->cat){
			pr_smkdebug(" ==================================Send-Action: [Send-Accept] Send-Netcard-level: [%x] Send-IP-level: [%x]\n",outdev->nic_secmark->level,ip_sec);//发送成功的标志
			return NF_ACCEPT;
		}	
		send_err = 4;
		pr_smkdebug(" ==================================Send-Action: [Send-Drop(cause of cat)] Send-Netcard-level: [%x] Send-IP-level: [%x]\n",outdev->nic_secmark->level,ip_sec_cat);//调试信息，类别不匹配导致的发送失败
		if (dfport != 5353 && dfport != 5354 && dfport != 5355 && dfport != 137 && dfport != 138 && dfport != 53)
			pr_info("transferlog         netcard=[%s]        user=[%u]        action=[send]        netcard_ifindex=[%d]        netcard_label=[%s]        ip_cipso=[%s]        protocol=[%s]        destaddr=[%d.%d.%d.%d]        destport=[%d]       result=[fail]        error_num=[%d]\n",devname,current->loginuid.val,ifindex,secmark,net_label,protocol,NIPQUAD(sdaddr),dfport,send_err);		
//		pr_info("transferlog         %s        %u        s        %d        netcardSM        %s        ip_data        %s        0         %s        %d.%d.%d.%d        %d.%d.%d.%d        %d        %d        fail %d\n",devname,current->loginuid.val,ifindex,secmark,net_label,protocol,NIPQUAD(ssaddr),NIPQUAD(sdaddr),sfport,dfport,send_err);//日志信息过于繁琐，已更新至新的日至格式		
//		kfree_secmark_struct(netcard_sec,ifindex);//释放网卡label结构体，现已不使用
		return NF_DROP;
       }
}

static unsigned int smack_ipv4_postroute(void *priv,
                                           struct sk_buff *skb,
                                           const struct nf_hook_state *state)
{
	if(!smack_enabled_my)
        	return NF_ACCEPT;
        return smack_ip_postroute(skb, state->out, PF_INET);
}

static struct nf_hook_ops smack_nf_ops[] = {
      {
		.hook =		smack_ipv4_postroute,
		.pf =		NFPROTO_IPV4,
		.hooknum =	NF_INET_POST_ROUTING,
		.priority =	NF_IP_PRI_SELINUX_FIRST,
	},
      /*{
		.hook =		smack_ipv4_input,
		.pf =		NFPROTO_IPV4,
		.hooknum =	NF_INET_LOCAL_IN,
		.priority =	NF_IP_PRI_SELINUX_FIRST,
	},*/ //input节点调取数据信息，现已不使用
	{
		.hook =		smack_ipv4_output,
		.pf =		NFPROTO_IPV4,
		.hooknum =	NF_INET_LOCAL_OUT,
		.priority =	NF_IP_PRI_SELINUX_FIRST,
	},
#if IS_ENABLED(CONFIG_IPV6)
	{
		.hook =		smack_ipv6_output,
		.pf =		NFPROTO_IPV6,
		.hooknum =	NF_INET_LOCAL_OUT,
		.priority =	NF_IP6_PRI_SELINUX_FIRST,
	},
#endif	/* IPV6 */
};

static int __net_init smack_nf_register(struct net *net)
{
	return nf_register_net_hooks(net, smack_nf_ops,
				     ARRAY_SIZE(smack_nf_ops));
}

static void __net_exit smack_nf_unregister(struct net *net)
{
	nf_unregister_net_hooks(net, smack_nf_ops, ARRAY_SIZE(smack_nf_ops));
}

static struct pernet_operations smack_net_ops = {
	.init = smack_nf_register,
	.exit = smack_nf_unregister,
};

static int __init smack_nf_ip_init(void)
{
	if (smack_enabled == 0)
		return 0;

	printk(KERN_DEBUG "Smack: Registering netfilter hooks\n");
	return register_pernet_subsys(&smack_net_ops);
}

__initcall(smack_nf_ip_init);
